unit TFlatProgressBarUnit;

{
  FlatStyle version 1.0
  Copyright (C) 1999 by Maik Porkert

  E-Mail:    maik.porkert@planet-interkom.de
  Homepage:  http://www.planet-interkom.de/maik.porkert
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Comctrls;

type
  TFlatProgressBar = class(TGraphicControl)
  private
    FOrientation: TProgressBarOrientation;
    FElementWidth: Integer;
    FElementColor: TColor;
    FBorderColor: TColor;
    FPosition: Integer;
    FMin: Integer;
    FMax: Integer;
    FStep: Integer;
    procedure SetMin(Value: Integer);
    procedure SetMax(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetStep(Value: Integer);
    procedure SetColor(Index: Integer; Value: TColor);
    procedure SetOrientation(Value: TProgressBarOrientation);
    procedure CheckBounds;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure StepIt;
    procedure StepBy(Delta: Integer);
  published
    property Cursor;
    property Color default $00E1EAEB;
    property ColorElement: TColor index 0 read FElementColor write SetColor default $00996633;
    property ColorBorder: TColor index 1 read FBorderColor write SetColor default $008396A0;
    property Orientation: TProgressBarOrientation read FOrientation write SetOrientation default pbHorizontal;
    property Enabled;
    property ParentColor;
    property Visible;
    property Hint;
    property ShowHint;
    property PopupMenu;
    property ParentShowHint;
    property Min: Integer read FMin write SetMin;
    property Max: Integer read FMax write SetMax;
    property Position: Integer read FPosition write SetPosition default 0;
    property Step: Integer read FStep write SetStep default 10;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

implementation

procedure TFlatProgressBar.SetOrientation(Value: TProgressBarOrientation);
begin
  if FOrientation <> Value then
  begin
    FOrientation := Value;
    if (csLoading in ComponentState) then
    begin
      Repaint;
      Exit;
    end;
    if FOrientation = pbVertical then
      SetBounds(Left, Top, 16, Width)
    else
      SetBounds(Left, Top, Height, 16);
    Repaint;
  end;
end;

procedure TFlatProgressBar.SetMin(Value: Integer);
begin
  if FMin <> Value then
  begin
    FMin := Value;
    Repaint;
  end;
end;

procedure TFlatProgressBar.SetMax(Value: Integer);
begin
  if FMax <> Value then
  begin
    FMax := Value;
    Repaint;
  end;
end;

procedure TFlatProgressBar.SetPosition(Value: Integer);
begin
  if FPosition <> Value then
  begin
    FPosition := Value;
    Repaint;
  end;
end;

procedure TFlatProgressBar.SetStep(Value: Integer);
begin
  if FStep <> Value then
  begin
    FStep := Value;
    Repaint;
  end;
end;

procedure TFlatProgressBar.StepIt;
begin
  if (FPosition + FStep) > FMax then
    FPosition := FMax
  else
    FPosition := FPosition + FStep;
  Repaint;
end;

procedure TFlatProgressBar.StepBy(Delta: Integer);
begin
  if (FPosition + Delta) > FMax then
    FPosition := FMax
  else
    FPosition := FPosition + Delta;
  Repaint;
end;

procedure TFlatProgressBar.SetColor(Index: Integer; Value: TColor);
begin
  case Index of
    0: FElementColor := Value;
    1: FBorderColor := Value;
  end;
  Invalidate;
end;

constructor TFlatProgressBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  Height := 16;
  Width := 147;
  FElementWidth := 8;
  FElementColor := $00996633;
  FBorderColor := $008396A0;
  ParentColor := True;
  Orientation := pbHorizontal;
  FStep := 10;
  FMin := 0;
  FMax := 100;
end;

procedure TFlatProgressBar.CheckBounds;
var
  maxboxes: Word;
begin
  if FOrientation = pbHorizontal then
  begin
    maxboxes := (Width - 3) div (FElementWidth + 1);
    if Width < 12 then
      Width := 12
    else
      Width := maxboxes * (FElementWidth + 1) + 3;
    SetBounds(Left, Top, Width, 16);
  end
  else
  begin
    maxboxes := (Height - 3) div (FElementWidth + 1);
    if Height < 12 then
      Height := 12
    else
      Height := maxboxes * (FElementWidth + 1) + 3;
    SetBounds(Left, Top, 16, Height);
  end;
end;

procedure TFlatProgressBar.Paint;
var
  NumElements: LongInt;
  PercentPerElement: Double;
  NumToPaint: LongInt;
  Painted: Integer;
  PaintRect, ElementRect: TRect;
begin
  CheckBounds;
  PaintRect := ClientRect;

  // Background
  Canvas.Brush.Color := Self.Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.FillRect(PaintRect);

  // Border
  Canvas.Brush.Color := FBorderColor;
  Canvas.FrameRect(PaintRect);

  // Elements
  if FOrientation = pbHorizontal then
  begin
    NumElements := Trunc((Width - 3) div (FElementWidth + 1));
    PercentPerElement := 100 div NumElements;
    NumToPaint := Round(FPosition / PercentPerElement);
    if NumToPaint > NumElements then
      NumToPaint := NumElements;
    ElementRect := Rect(PaintRect.Left + 2, PaintRect.Top + 2, PaintRect.Left + 2 + FElementWidth, PaintRect.Bottom - 2);

    if NumToPaint > 0 then
    begin
      Canvas.Brush.Color := FElementColor;
      Canvas.Brush.Style := bsSolid;
      for Painted := 1 to NumToPaint do
      begin
        Canvas.FillRect(ElementRect);
        ElementRect.Left := ElementRect.Left + FElementWidth + 1;
        ElementRect.Right := ElementRect.Right + FElementWidth + 1;
      end;
    end;
  end
  else
  begin
    NumElements := Trunc((Height - 3) div (FElementWidth + 1));
    PercentPerElement := 100 div NumElements;
    NumToPaint := Round(FPosition / PercentPerElement);
    if NumToPaint > NumElements then
      NumToPaint := NumElements;
    ElementRect := Rect(PaintRect.Left + 2, PaintRect.Bottom - FElementWidth - 2, PaintRect.Left + 12 + 2, PaintRect.Bottom - 2);

    if NumToPaint > 0 then
    begin
      Canvas.Brush.Color := FElementColor;
      Canvas.Brush.Style := bsSolid;
      for Painted := 1 to NumToPaint do
      begin
        Canvas.FillRect(ElementRect);
        ElementRect.Top := ElementRect.Top - (FElementWidth + 1);
        ElementRect.Bottom := ElementRect.Bottom - (FElementWidth + 1);
      end;
    end;
  end;
end;

end.
